-- The ZIG Editor, version 1.0.
--
-- ZIG v1.0 990814
-- Copyright 1999, Jacob Hammond.
-- Webpage: http://surf.to/zig
--          or http://www2.crosswinds.net/~zig/index.htm
-- E-mail:  zig@crosswinds.net
--

include graphics.e
include machine.e
include image.e
include get.e
include file.e
include misc.e

include modwave.e
include zfades.e
include palette.e
include keyread.e
-- scroll down for more includes


without trace
with type_check     -- remove sometime

puts(1, "\n\nThis is the ZIG Editor, version 1.0.\n(C) 1997-99, Jacob Hammond.\n")

-- DOS Version Detect Routine
atom Ver, SubVer, x
sequence reg_list, Dir
reg_list = repeat(0, 10)
reg_list[REG_AX] = #3000
reg_list = dos_interrupt(#21, reg_list)
if and_bits(reg_list[REG_FLAGS], 1) then
    puts(1, "WARNING: DOS Version detect failed. Report error code ST001.\n")
end if
Ver = remainder(reg_list[REG_AX], #100)
SubVer = floor(reg_list[REG_AX] / #100)
Dir = current_dir()
text_color(7)
if Ver = 7 then
    if SubVer != 10 then puts(1, "Windows 95 and DOS 7.0 present.\n")
    else puts(1, "Windows 98 and DOS 7.1 present.\n")
    end if
else printf(1, "DOS %1d.%1d detected.\n", {Ver, SubVer})
end if
if platform() = DOS32 then puts(1, "Running under DOS32.\n")
else printf(1, "Not running under DOS32 (platform code %d). This may cause errors...\n", platform())
end if
puts(1, "Current directory: " & Dir & "\n\nPress any key to continue...")
Ver = wait_key()

    
-- Global Constants ---------------------------------------------------------

global constant GRAB_OBJ_SEL = 18

-- Global variables ---------------------------------------------------------

-- Current colors for editor - fore, back, single-atom combo
global atom CColorF, CColorB, CColor


global sequence CustTiles, SSBuffer2, CharSet, DefaultCharSet, DefaultPalette

-- Predefined custom tiles for editor tile chooser, plus blanks
CustTiles = 219 & 178 & 177 & 176 & 254 & repeat(0, 12)

include zchar.e
include zlibrs.e
include zobjects.e
include zboards.e
include zgfx.e
include zworlds.e
include zsidebar.e
include zchared.e
include zgui.e
include zutil.e
include zsie.e
include zfill.e
include ztxfr.e

-- Define boards (OneBoard stuck onto/into Boards for new board)
--OneBoard = repeat(repeat(repeat({0,0,0}, 3), 80), 25) <<old def
OneBoard = repeat(0, BOARD_ELEMS)
Boards = {OneBoard}

-- World and board properties, using W_PROPS / B_PROPS
WorldProps = repeat({0}, length(W_PROPS))
WorldProps[WP_TBOARD][1] = 1
WorldProps[WP_SBOARD][1] = 1
BoardProps = repeat(repeat({0}, length(B_PROPS)), 1)

-- Object definitions, and object position table
           -- boardn,x,y,layer,char,col,dmode,pos,state,locked,cycle,name,hasp,props x 4
OneObject = {{   0,    0,0,  0,   0,  00,   0,   0,   0,     0,    0,   {},  0,  0,0,0,0}}
Objects = {}
GrabbedObject = OneObject
ObjectPosTable = repeat(repeat(repeat(0, 3), 80), 25)


-- Procedures and Functions -------------------------------------------------


procedure Init()
    atom f
    sequence LibNames, dx

    if graphics_mode(3) then
        puts(1, "\nERROR:\nCouldn't get into videomode 3! (text 80x25x8x8)\nReport error code GE_INIT_01.\n")
        EndProg(ERR_INIT_ERROR)
    end if
    
    dx = {}
    for z = 0 to 15 do
        dx = append(dx, get_palette(z))
    end for
    
    egachars()
    allow_all_paledit()
    cursor(NO_CURSOR)
    toggle_blinking(0)
    ZIGSwitchPal(dx)
    DefaultPalette = dx
    ZIGLoadFont("default.zch")
    DefaultCharSet = CharSet
    
    BoardProps[1][BP_CHARSET] = DefaultCharSet
    BoardProps[1][BP_PAL] = DefaultPalette

    CurBoard = 1        
    
    Scroll_SizeX = 16
    Scroll_SizeY = 50
    Scroll_PosX = 5
    Scroll_PosY = 10
    Scroll_TextC = 14
    Scroll_BackC = 1
    Scroll_HLTitleC = 15
    Scroll_BorderC = 9
    
    if sequence(dir("zig_libs.inf")) then
        f = open("zig_libs.inf", "r")
    else
        f = open("zig_libs.inf", "w")
        if length(dir("stdlib.olf")) then
            puts(f, "stdlib.olf")
        else puts(f, "NULL")
        end if
        close(f)
        f = open("zig_libs.inf", "r")
    end if
    
    LibNames = {}
    dx = {}
    while TRUE do
        dx = gets(f)
        dx = dx[1..length(dx) - 1]
        if not compare(dx, "NULL") then
            exit
        end if
        if length(dir(dx)) then
            LibNames = append(LibNames, dx)
        end if
    end while
    close(f)

    ObjectLibrary = LoadLibraries({}, LibNames)
end procedure

procedure DrawBoardName(atom Board)
    position(1, 1)
    color(14, 8)
    puts(1, ' ' & BoardProps[Board][BP_NAME] & ' ')
end procedure

procedure Edit_Main()
    atom X, Y, Key, OldX, OldY, Layer, FWMode, PatPos, OldPP,
        Timer, DrawMode, R1, R2, SelX1, SelY1, SelX2, SelY2, Sel, BNTime
    sequence EdVisLayers, OldEVLs, SS, SelO
    object Dummy1, Dummy2, Dummy3, Dummy4, PKey
    
    CColor = 15
    CColorF = 15
    CColorB = 0
    Layer = 2
    FWMode = 0
    PatPos = 1
    X = 12
    Y = 30
    Timer = 0
    DrawMode = FALSE
    SelX1 = -1
    SelY1 = -1
    SelX2 = -1
    SelY2 = -1
    Sel = FALSE
    
    R1 = 1
    R2 = length(ObjectLibrary)
    if R2 > 16 then R2 = 16 end if
        
    EdVisLayers = {1, 1, 1}
    
    SB_M_Draw()
    DrawBoard_Ed(CurBoard, EdVisLayers)    
    
    BNTime = time() + 2
    
    while 1 do
        
        if BNTime != 0 then
            if time() > BNTime then
                BNTime = 0
                DrawBoard1L(CurBoard, 1, EdVisLayers)
            end if
        end if
        Timer = Timer + 1
        if Timer > 8000 then Timer = 0 end if
        
        PKey = get_key()
        Key = upper(PKey)
        
        OldX = X
        OldY = Y
        OldEVLs = EdVisLayers
        SelO = {SelX1, SelY1, SelX2, SelY2}
                
        if Key = KEY_UP then
            X = X - 1
        elsif Key = KEY_DN then
            X = X + 1
        elsif Key = KEY_LF then
            Y = Y - 1
        elsif Key = KEY_RT then
            Y = Y + 1
        elsif Key = KEY_TAB then
            if DrawMode != FALSE then
                DrawMode = FALSE
            else DrawMode = TRUE
            end if
            SB_M_DrawDrawStat(DrawMode)
        elsif Key = KEY_F2 then
            if DrawMode != 2 then
                DrawMode = 2
            else DrawMode = FALSE
            end if
            SB_M_DrawDrawStat(DrawMode)
        elsif Key = KEY_HOME then
            if X != 1 then
                X = 1
            else Y = 1
            end if
        elsif Key = KEY_END then
            if X != 25 then
                X = 25
            else Y = 60
            end if            
        elsif Key = KEY_ALT_UP then
            X = X - 5
        elsif Key = KEY_ALT_DN then
            X = X + 5
        elsif Key = KEY_ALT_LF then
            Y = Y - 5
        elsif Key = KEY_ALT_RT then
            Y = Y + 5
        elsif Key = KEY_ESC then
            if DrawMode = 0 then
                EndProg(ERR_SAFE_PRG_EXIT)
            else
                DrawMode = 0
                SB_M_DrawDrawStat(DrawMode)
            end if            
        end if
        
        if DrawMode = 2 then
            if PKey != NULL then
                if PKey > 31 and PKey < 127 then
                    Boards[CurBoard][BoardI(X, Y, Layer, 1)] = 0
                    Boards[CurBoard][BoardI(X, Y, Layer, 2)] = PKey
                    Boards[CurBoard][BoardI(X, Y, Layer, 3)] = ColAttrib(CColorF, CColorB)
--                    Boards[CurBoard][X][Y][Layer] =
--                                {0, PKey, ColAttrib(CColorF, CColorB)}                                
                    if Y < 61 then
                        Y = Y + 1
                    end if
                else
                    if PKey = 8 then
                        if Y > 0 then
                            Y = Y - 1
                            Boards[CurBoard][BoardI(X, Y, Layer, 1)] = 0
                            Boards[CurBoard][BoardI(X, Y, Layer, 2)] = 0
                            Boards[CurBoard][BoardI(X, Y, Layer, 3)] = 0
                        end if
                    end if
                end if
            end if
        else                
            if Key = 'P' then
                OldPP = PatPos
                PatPos = PatPos + 1
                if PatPos > length(CustTiles) + 1 then
                    PatPos = 1
                end if
                SB_M_DrawCTile(PatPos, OldPP)
            elsif Key = 281 then        -- Alt+P
                OldPP = PatPos
                PatPos = PatPos - 1
                if PatPos < 1 then
                    PatPos = GRAB_OBJ_SEL
                end if
                SB_M_DrawCTile(PatPos, OldPP)
            elsif Key = 'Z' then
                if FWMode then FWMode = 0 else FWMode = 1 end if
                SB_M_DrawFWP(FWMode)
            elsif Key = '~' or Key = '`' then
                Layer = Layer + 1
                if Layer = 4 then Layer = 1 end if
                SB_M_DrawLayer(Layer)
            elsif Key = '1' then
                if EdVisLayers[1] then EdVisLayers[1] = 0 else EdVisLayers[1] = 1                
                end if
                SB_M_DrawShowns(EdVisLayers)
            elsif Key = '2' then
                if EdVisLayers[2] then EdVisLayers[2] = 0 else EdVisLayers[2] = 1                
                end if
                SB_M_DrawShowns(EdVisLayers)
            elsif Key = '3' then
                if EdVisLayers[3] then EdVisLayers[3] = 0 else EdVisLayers[3] = 1
                end if
                SB_M_DrawShowns(EdVisLayers)
           elsif Key = ' ' then
                if PatPos != GRAB_OBJ_SEL then
                    if ObjectPosTable[X][Y][Layer] != 0 then
                        -- Remember... DeleteObject also removes it from the table
                        DeleteObject(ObjectPosTable[X][Y][Layer], CurBoard)
                    else
                    if Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = 0 then
                        Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = CustTiles[PatPos]
                        Boards[CurBoard][BoardI(X, Y, Layer, COL)] = ColAttrib(CColorF, CColorB)
                        Boards[CurBoard][BoardI(X, Y, Layer, NUM)] = FWMode
                    else
                        if ObjectPosTable[X][Y][Layer] != 0 then
                            DeleteObject(ObjectPosTable[X][Y][Layer], CurBoard)
                        end if
                        Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = 0
                    end if
                    end if
                else
                    if ObjectPosTable[X][Y][Layer] != 0 then
                        DeleteObject(ObjectPosTable[X][Y][Layer], CurBoard)
                    else
                        Boards[CurBoard][BoardI(X, Y, Layer, NUM)] = 0
                        Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = 0
                        Boards[CurBoard][BoardI(X, Y, Layer, COL)] = 0
                        Dummy1 = GrabbedObject
                        Dummy1[1][OBJ_X] = X
                        Dummy1[1][OBJ_Y] = Y
                        Dummy1[1][OBJ_LAYER] = Layer
                        Dummy1[1][OBJ_BOARD_NUM] = CurBoard
                        NewObject(Dummy1[1], TRUE)
                        if length(Dummy1) > 1 then
                            for z = 2 to length(Dummy1) do
                                Objects[length(Objects)] =
                                        append(Objects[length(Objects)],
                                        Dummy1[z])
                            end for
                        end if
                    end if
                    DrawBoard_Ed(CurBoard, EdVisLayers)
                end if
            elsif Key = 'D' then
                CustTiles[PatPos] = GetChar(CustTiles[PatPos])
                SB_M_DrawCustTs()
            elsif Key = 'C' then
                ColorDialog()
            elsif Key = 'S' then                
                Dummy1 = WriteWorld(
                        SB_Input(SBI_FILENAME, 21, "Filename?ZIG", 0) & ".ZIG")
            elsif Key = 'L' then
                Dummy1 = LoadWorld(ChooseFile("*.zig"))
                if length(WorldProps[WP_TBOARD]) != 0 then
                    CurBoard = WorldProps[WP_TBOARD][1]
                else CurBoard = 1
                end if
                DrawBoard_Ed(CurBoard, EdVisLayers)
            elsif Key = KEY_CT_UP
            or Key = KEY_CT_DN
            or Key = KEY_CT_LF
            or Key = KEY_CT_RT then
                if Sel = FALSE then
                    Sel = TRUE
                    SelX1 = X
                    SelY1 = Y
                    SelX2 = X
                    SelY2 = Y
                else
                    if Key = KEY_CT_UP then
                        SelX2 = SelX2 - 1
                        X = X - 1
                    elsif Key = KEY_CT_DN then
                        SelX2 = SelX2 + 1
                        X = X + 1
                    elsif Key = KEY_CT_LF then
                        SelY2 = SelY2 - 1
                        Y = Y - 1
                    elsif Key = KEY_CT_RT then
                        SelY2 = SelY2 + 1
                        Y = Y + 1
                    end if
                end if
            elsif Key = KEY_F1 then

                SS = SaveScreen()
                
                while TRUE do
                    SB_Clear()
        
                    Box(3, 61, 20, 80, 1, "Choose an Object")
                    Dummy1 = repeat(0, length(ObjectLibrary) + 1)
                    Dummy3 = 3
                    SB_Text(1, 15, "Building key")
                    SB_Text(2, 15, "table...")
                    for z = R1 to R2 do
                        Dummy2 = 1
                        while TRUE do
                            Dummy1[z] = upper(ObjectLibrary[z][LIB_NAME][Dummy2])
                            Dummy1[length(ObjectLibrary) + 1] = OFF                        
                            if Dummy1[z] = ' ' then
                                Dummy1[z] = Dummy1[R1]
                            end if
                            for a = R1 to z - 1 do
                                if Dummy1[a] = Dummy1[z] then
                                    Dummy2 = Dummy2 + 1
                                    if Dummy2 > length(ObjectLibrary[z][LIB_NAME]) then
                                        Dummy1[length(Dummy1)] = ON
                                        for b = 'A' to 'Z' do
                                            for c = R1 to z - 1 do
                                                if Dummy1[c] = b then
                                                    Dummy1[length(Dummy1)] = OFF
                                                    exit
                                                else Dummy1[length(Dummy1)] = ON
                                                end if
                                            end for
                                            if Dummy1[length(Dummy1)] = ON then
                                                Dummy1[length(Dummy1)] = 2
                                                Dummy1[z] = b
                                                exit
                                            end if
                                        end for                                    
                                        if Dummy1[length(Dummy1)] = OFF then
                                            Dummy1[length(Dummy1)] = ON
                                            for b = '0' to '9' do
                        

                                            end for
                                        end if
                                    end if
                                    if Dummy1[length(Dummy1)] = 2 then
                                        Dummy1[length(Dummy1)] = OFF
                                        exit
                                    else
                                        Dummy1[length(Dummy1)] = ON
                                        exit
                                    end if
                                end if
                            end for                        
                            if Dummy1[length(Dummy1)] = OFF then
                                exit
                            end if                        
                        end while
                        SB_BtnText(3 + (z - (R1 - 1)), Dummy3, " " & {Dummy1[z]} & "  " & ObjectLibrary[z][LIB_NAME])
                        Dummy3 = MBColRotate(Dummy3)
                    end for
                    SB_Text(1, 15, "            ")
                    SB_Text(2, 15, "        ")
            
                    SB_BtnText(21, Dummy3, " PU  Move list up")
                    SB_BtnText(22, MBColRotate(Dummy3), " PD  Move list down")
                    SB_BtnText(23, Dummy3, " F1  Auto#bind:Off")
                
                    Dummy2 = FALSE
                    Dummy4 = FALSE
                    while TRUE do
                    
                        Key = get_key()
                    
                        if Key != -1 then
                            if Key = KEY_F1 then
                                if Dummy2 = FALSE then
                                    Dummy2 = TRUE
                                    SB_BtnText(23, Dummy3, " F1  Auto#bind:On ")
                                else
                                    Dummy2 = FALSE
                                    SB_BtnText(23, Dummy3, " F1  Auto#bind:Off")
                                end if                                
                            elsif Key = KEY_PGUP then
                                Dummy4 = R1
                                R1 = R1 - 15
                                
                                if R1 < 1 then
                                    R1 = 1
                                end if
                                
                                if Dummy4 != R1 then
                                    R2 = R1 + 15
                                    Dummy4 = TRUE
                                    exit
                                else
                                    Dummy4 = FALSE
                                end if
                            elsif Key = KEY_PGDN then
                                Dummy4 = R1
                                R1 = R1 + 15
                                
                                if R1 + 15 > length(ObjectLibrary) then
                                    R1 = Dummy4
                                end if
                                
                                if Dummy4 != R1 then
                                    R2 = R1 + 15
                                    Dummy4 = TRUE
                                    exit
                                else
                                    Dummy4 = FALSE
                                end if                                
                            elsif Key = KEY_CR or Key = KEY_ESC then
                                exit
                            else
                                Key = upper(Key)
                                for z = R1 to R2 do
                                    Dummy1[length(Dummy1)] = FALSE
                                    if Key = Dummy1[z] then
                                        Boards[CurBoard][BoardI(X, Y, Layer, NUM)] = 0
                                        Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = 0
                                        Boards[CurBoard][BoardI(X, Y, Layer, COL)] = 0
                                        if ObjectPosTable[X][Y][Layer] != 0 then
                                            DeleteObject(ObjectPosTable[X][Y][Layer], CurBoard)
                                        end if
                                        NewObject(
                                            {CurBoard,X,Y,Layer,0,CColor, 1,   1,   1,     0,    1,   {},  0,  0,0,0,0},
                                            TRUE)
                                        Objects[length(Objects)] = ModifyObjectProps(Objects[length(Objects)], ObjectLibrary[z])
                                        Objects[length(Objects)] = Objects[length(Objects)] & ObjectLibrary[z][LIB_PROG]
                                        if ObjectLibrary[z][LIB_CHAR] = -1 then
                                            Objects[length(Objects)][1][OBJ_CHAR] = GetChar(0)
                                        else
                                            Objects[length(Objects)][1][OBJ_CHAR] = ObjectLibrary[z][LIB_CHAR]
                                        end if
                                        DrawBoard_Ed(CurBoard, EdVisLayers)
                                        CColor = Objects[ObjectPosTable[X][Y][Layer]][1][OBJ_COLOR]
                                        CColorF = CGetFore(CColor)
                                        CColorB = CGetBack(CColor)
                                        GrabbedObject = Objects[ObjectPosTable[X][Y][Layer]]
                                        OldPP = PatPos
                                        PatPos = GRAB_OBJ_SEL
                                        Dummy1[length(Dummy1)] = TRUE
                                        exit
                                    end if
                                end for
                                if Dummy1[length(Dummy1)] then
                                    exit
                                end if
                            end if                        
                        end if
                    end while
                    if Dummy4 = FALSE then exit end if
                end while
                
                ResScreen(SS)
                DrawBoard_Ed(CurBoard, EdVisLayers)
                SB_M_Draw()
                SB_M_DrawCTile(GRAB_OBJ_SEL, OldPP)
                SB_M_DrawCol(ColAttrib(CColorF, CColorB))
                SB_M_DrawFWP(FWMode)
                SB_M_DrawGrabbedObj(GrabbedObject)
                SB_M_DrawLayer(Layer)
                SB_M_DrawShowns(EdVisLayers)
                
            elsif Key = 'O' then
           -- boardn,x,y,layer,char,col,dmode,pos,state,locked,cycle,name,hasp,props x 4
                Boards[CurBoard][BoardI(X, Y, Layer, NUM)] = 0
                Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = 0
                Boards[CurBoard][BoardI(X, Y, Layer, COL)] = 0
                NewObject(
            {CurBoard,X,Y,Layer,GetChar(0),CColor, 1,   1,   1,     0,    1,   {},  0,  0,0,0,0},
                TRUE)
                DrawBoard_Ed(CurBoard, EdVisLayers)
                Objects[length(Objects)] = sie_Ed(Objects[length(Objects)])
                CColor = Objects[ObjectPosTable[X][Y][Layer]][1][OBJ_COLOR]
                CColorF = CGetFore(CColor)
                CColorB = CGetBack(CColor)
                SB_M_DrawCol(CColor)
                GrabbedObject = Objects[ObjectPosTable[X][Y][Layer]]
                SB_M_DrawGrabbedObj(GrabbedObject)
                OldPP = PatPos
                PatPos = GRAB_OBJ_SEL
                SB_M_DrawCTile(PatPos, OldPP)
            elsif Key = KEY_CR then
                if ObjectPosTable[X][Y][Layer] = 0 then
                    CColor = Boards[CurBoard][BoardI(X, Y, Layer, COL)]
                    CColorF = CGetFore(CColor)
                    CColorB = CGetBack(CColor)
                    SB_M_DrawCol(CColor)
                    OldPP = PatPos
                    Dummy1 = Boards[CurBoard][BoardI(X, Y, Layer, CHAR)]
                    Dummy2 = 1
                    for z = 1 to length(CustTiles) do
                        Dummy2 = Dummy2 + 1
                        if CustTiles[z] = Dummy1 then
                            PatPos = z
                            exit
                        end if
                    end for
                    if Dummy2 > length(CustTiles) then
                        Dummy2 = 1
                        for z = 1 to length(CustTiles) do
                            if CustTiles[z] = 0 then
                                CustTiles[z] = Dummy1
                                PatPos = z
                                exit
                            end if
                        end for
                        if Dummy2 > length(CustTiles) then
                            CustTiles[length(CustTiles)] = Dummy1
                            PatPos = length(CustTiles)
                        end if
                    end if
                    SB_M_DrawCustTs()
                    SB_M_DrawCTile(PatPos, OldPP)
                else
                    CColor = Objects[ObjectPosTable[X][Y][Layer]][1][OBJ_COLOR]
                    CColorF = CGetFore(CColor)
                    CColorB = CGetBack(CColor)
                    SB_M_DrawCol(CColor)
                    GrabbedObject = Objects[ObjectPosTable[X][Y][Layer]]
                    SB_M_DrawGrabbedObj(GrabbedObject)
                    OldPP = PatPos
                    PatPos = GRAB_OBJ_SEL
                    SB_M_DrawCTile(PatPos, OldPP)                    
                    Objects[ObjectPosTable[X][Y][Layer]] = {Objects[ObjectPosTable[X][Y][Layer]][1]} &
                        sie_Ed(Objects[ObjectPosTable[X][Y][Layer]][2..length(Objects[ObjectPosTable[X][Y][Layer]])])
                    --Objects[ObjectPosTable[X][Y][Layer]] = sie_Ed(Objects[ObjectPosTable[X][Y][Layer]])
                end if
            elsif Key = 'W' then
                WorldProps = GetProps(W_PROPS, WorldProps, "World Properties", 0)
            elsif Key = 'I' then
                BoardProps[CurBoard] = GetProps(B_PROPS, BoardProps[CurBoard], "Board Properties", CurBoard)
            elsif Key = 'B' then
                OldPP = CurBoard
                CurBoard = GetBoard(CurBoard)
                if CurBoard = 0 then
                    CurBoard = OldPP
                end if
                if CurBoard = 9999 then
                    Boards = append(Boards, OneBoard)
                    BoardProps = append(BoardProps, repeat({}, length(B_PROPS)))
                    CurBoard = length(Boards)
                    BoardProps[CurBoard][BP_CHARSET] = DefaultCharSet
                    BoardProps[CurBoard][BP_PAL] = DefaultPalette
                end if
                if CurBoard != OldPP then
                        SB_M_DrawCBoard(CurBoard)
                        BuildObjectTable(CurBoard)
                        if length(BoardProps[CurBoard][BP_CHARSET]) != 0 then
                            CharSet = BoardProps[CurBoard][BP_CHARSET]
                            add_all_fonts(0, CharSet)
                        end if
                        if length(BoardProps[CurBoard][BP_PAL]) != 0 then
                            ZIGSwitchPal(BoardProps[CurBoard][BP_PAL])
                        end if
                        DrawBoard_Ed(CurBoard, EdVisLayers)
                        BNTime = time() + 2
                end if
            elsif Key = 'T' then
                Dummy1 = DoTransfer(CurBoard)
            elsif Key = KEY_PGUP then
                OldPP = CurBoard
                CurBoard = CurBoard - 1                
                if CurBoard < 1 then CurBoard = 1 end if        
                if CurBoard != OldPP then
                    BuildObjectTable(CurBoard)
                    SB_M_DrawCBoard(CurBoard)
                    DrawBoard_Ed(CurBoard, EdVisLayers)
                    if length(BoardProps[CurBoard][BP_CHARSET]) != 0 then
                        CharSet = BoardProps[CurBoard][BP_CHARSET]
                        add_all_fonts(0, CharSet)
                    end if
                    if length(BoardProps[CurBoard][BP_PAL]) != 0 then
                        ZIGSwitchPal(BoardProps[CurBoard][BP_PAL])
                    end if
                    DrawBoardName(CurBoard)
                    BNTime = time() + 2
                end if
            elsif Key = KEY_PGDN then
                OldPP = CurBoard
                CurBoard = CurBoard + 1
                if CurBoard > length(Boards) then CurBoard = length(Boards) end if
                if CurBoard != OldPP then
                    SB_M_DrawCBoard(CurBoard)
                    BuildObjectTable(CurBoard)
                    DrawBoard_Ed(CurBoard, EdVisLayers)
                    if length(BoardProps[CurBoard][BP_CHARSET]) != 0 then
                        CharSet = BoardProps[CurBoard][BP_CHARSET]
                        add_all_fonts(0, CharSet)
                    end if
                    if length(BoardProps[CurBoard][BP_PAL]) != 0 then
                        ZIGSwitchPal(BoardProps[CurBoard][BP_PAL])
                    end if
                    DrawBoardName(CurBoard)
                    BNTime = time() + 2
                end if
            elsif Key = 'X' then
                FillArea(
                    {X, Y},
                    FWMode & CustTiles[PatPos] & ColAttrib(CColorF, CColorB),
                    Boards[CurBoard][BoardI(X, Y, Layer, NUM)] &
                    Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] &
                    Boards[CurBoard][BoardI(X, Y, Layer, COL)],
                    CurBoard, Layer)
            end if
        end if
                                             
        if compare(EdVisLayers, OldEVLs) then
            DrawBoard_Ed(CurBoard, EdVisLayers)
        end if
        
        if Timer = 2000 then
            DrawTile(CurBoard, OldX, OldY, {1,1,1})
        end if

        if X != OldX or Y != OldY then
            if X < 1 then X = 1 SelX1 = SelO[1] end if
            if Y < 1 then Y = 1 SelY1 = SelO[2] end if
            if X > 25 then X = 25 SelX2 = SelO[3] end if
            if Y > 60 then Y = 60 SelY2 = SelO[4] end if
            if compare(SelO, {SelX1, SelY1, SelX2, SelY2}) = 0
            and Sel = TRUE then
                Sel = FALSE
                SelX1 = -2
            end if
            position(22, 62)
            text_color(14)
            bk_color(1)
            printf(1, "(%2d, %2d)", {X, Y})
        end if
        
        if DrawMode = TRUE and PatPos != GRAB_OBJ_SEL then
            Boards[CurBoard][BoardI(X, Y, Layer, CHAR)] = CustTiles[PatPos]
            Boards[CurBoard][BoardI(X, Y, Layer, COL)] = ColAttrib(CColorF, CColorB)
            Boards[CurBoard][BoardI(X, Y, Layer, NUM)] = FWMode
        end if
        
        if Sel != FALSE then
            if compare(SelO, {SelX1, SelY1, SelX2, SelY2}) != 0 then
                DrawBoard_Ed(CurBoard, EdVisLayers)
                InvertArea(SelX1, SelY1, SelX2, SelY2)
            end if
        else
            if SelX1 = -2 then
                DrawBoard(CurBoard, EdVisLayers)
                SelX1 = -1
            end if
        end if
        
        if X != OldX or Y != OldY or Timer = 0 then
            DrawTile(CurBoard, OldX, OldY, {1,1,1})
            position(X, Y)
            text_color(15)
            bk_color(DrawMode)
            if DrawMode = 2 then
                puts(1, '')
            else puts(1, '')
            end if
        end if
            
    end while
    
end procedure

Init()
Edit_Main()

